﻿using Infragistics.Win.UltraWinEditors;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DateRange
{
    public class DateRangePicker : UltraComboEditor
    {
        #region Private Members
        private Panel dropdownPanel;
        private TableLayoutPanel tableLayoutPanel;
        private Label lblMin;
        private Label lblMax;
        private MonthCalendar mcStart;
        private MonthCalendar mcEnd;
        #endregion // Private Members

        #region Private properties

        #region DropDownPanel
        private Panel DropDownPanel
        {
            get
            {
                if (null == this.dropdownPanel)
                {
                    this.dropdownPanel = new Panel();                    
                    this.dropdownPanel.Controls.Add(this.TableLayoutPanel);
                    this.dropdownPanel.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
                    this.dropdownPanel.Size = this.TableLayoutPanel.PreferredSize;
                }

                return this.dropdownPanel;
            }
        }
        #endregion // DropDownPanel

        #region McStart
        private MonthCalendar McStart
        {
            get
            {
                if (null == this.mcStart)
                    this.mcStart = this.CreateMonthCalendar("mcMin");

                return this.mcStart;
            }
        }

        #endregion // McStart

        #region McEnd
        private MonthCalendar McEnd
        {
            get
            {
                if (null == this.mcEnd)
                    this.mcEnd = this.CreateMonthCalendar("mcMax");

                return this.mcEnd;
            }
        }
        #endregion // McEnd

        #region LblMin
        private Label LblMin
        {
            get
            {
                if (null == this.lblMin)
                {
                    this.lblMin = new Label();
                    this.lblMin.Text = "Min Date";
                }

                return this.lblMin;
            }
        }
        #endregion // LblMin

        #region LblMax
        private Label LblMax
        {
            get
            {
                if (null == this.lblMax)
                {
                    this.lblMax = new Label();
                    this.lblMax.Text = "Max Date";
                }

                return this.lblMax;
            }
        }
        #endregion // LblMax

        #region TableLayoutPanel
        private TableLayoutPanel TableLayoutPanel
        {
            get
            {
                if (null == this.tableLayoutPanel)
                {
                    this.tableLayoutPanel = new TableLayoutPanel();
                    this.tableLayoutPanel.AutoSize = true;

                    this.tableLayoutPanel.Controls.Add(this.LblMin);
                    this.tableLayoutPanel.Controls.Add(this.LblMax);
                    this.tableLayoutPanel.Controls.Add(this.McStart);
                    this.tableLayoutPanel.Controls.Add(this.McEnd);

                    this.tableLayoutPanel.ColumnCount = 2;
                    this.tableLayoutPanel.RowCount = 2;

                    this.tableLayoutPanel.SetCellPosition(this.LblMin, new TableLayoutPanelCellPosition(0, 0));
                    this.tableLayoutPanel.SetCellPosition(this.LblMax, new TableLayoutPanelCellPosition(1, 0));
                    this.tableLayoutPanel.SetCellPosition(this.McStart, new TableLayoutPanelCellPosition(0, 1));
                    this.tableLayoutPanel.SetCellPosition(this.McEnd, new TableLayoutPanelCellPosition(1, 1));

                    this.Controls.Add(this.tableLayoutPanel);

                    this.UpdateValueAndText();
                }

                return this.tableLayoutPanel;
            }
        }
        #endregion // TableLayoutPanel

        #endregion // Private properties

        #region Public Properties

        #region StartDate
        public DateTime StartDate
        {
            get
            {
                return this.McStart.SelectionStart;
            }
            set
            {
                this.McStart.SelectionStart = value;
                this.UpdateValueAndText();
            }
        }
        #endregion // StartDate

        #region EndDate
        public DateTime EndDate
        {
            get
            {
                return this.McEnd.SelectionEnd;
            }
            set
            {
                this.McEnd.SelectionEnd = value;
                this.UpdateValueAndText();
            }
        }
        #endregion // EndDate

        #endregion // Public Properties

        #region Private Methods

        #region CreateMonthCalendar
        private MonthCalendar CreateMonthCalendar(string name)
        {
            var monthCalendar = new MonthCalendar();
            monthCalendar.Name = name;
            monthCalendar.MaxSelectionCount = 1;
            monthCalendar.DateSelected += mc_DateSelected;

            return monthCalendar;
        }
        #endregion // CreateMonthCalendar

        #region UpdateValueAndText
        private void UpdateValueAndText()
        {
            var dateRangeInfo = new DateRangeInfo(this.StartDate, this.EndDate);
            if (!IsItemInList(dateRangeInfo.ToString()))
            {
                this.ValueList.ValueListItems.Clear();
                this.ValueList.ValueListItems.Add(dateRangeInfo, dateRangeInfo.ToString());
            }
            this.Value = dateRangeInfo;
            
            this.McStart.MaxDate = this.McEnd.SelectionStart;
            this.McEnd.MinDate = this.McStart.SelectionStart;

            try
            {
                this.OnValueChanged(EventArgs.Empty);
            }
            catch (InvalidOperationException ex)
            {
            }
        }
        #endregion // UpdateValueAndText

        #region VerifyDropDownButtonContent
        private void VerifyDropDownButtonContent()
        {
            this.ButtonsRight.Clear();
            DropDownEditorButton editorButton = new DropDownEditorButton();
            editorButton.Control = this.DropDownPanel;
            this.ButtonsRight.Add(editorButton);
        }
        #endregion // VerifyDropDownButtonContent 

        #endregion // Private Methods

        #region Overrides

        #region OnCreateControl
        protected override void OnCreateControl()
        {
            base.OnCreateControl();

            this.DropDownStyle = Infragistics.Win.DropDownStyle.DropDownList;
            this.DropDownButtonDisplayStyle = Infragistics.Win.ButtonDisplayStyle.Never;

            this.VerifyDropDownButtonContent();
        }       
        #endregion // OnCreateControl

        #region Dispose
        protected override void Dispose(bool disposing)
        {
            if (null != this.tableLayoutPanel)
                this.tableLayoutPanel.Dispose();

            if (null != this.mcStart)
                this.mcStart.Dispose();

            if (null != this.mcEnd)
                this.mcEnd.Dispose();

            if (null != this.mcStart)
                this.mcStart.Dispose();

            if (null != this.mcEnd)
                this.mcEnd.Dispose();

            base.Dispose(disposing);
        }
        #endregion // Dispose

        #endregion // Overrides

        #region Event Handlers

        #region mc_DateSelected
        private void mc_DateSelected(object sender, DateRangeEventArgs e)
        {            
            this.UpdateValueAndText();
        }   
        #endregion // mc_DateSelected      

        #endregion // Event Handlers
    }
}
